﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;

namespace MikuMikuDance.XNA.Model
{
    /// <summary>
    /// MMD基本頂点データ
    /// </summary>
    [ContentSerializerRuntimeType("MikuMikuDance.XNA.Model.MMDVertex, MikuMikuDanceXNA")]
    public class MMDVertexContent
    {
        /// <summary>
        /// 位置
        /// </summary>
        public Vector3 Position;
        /// <summary>
        /// ボーンウェイト
        /// </summary>
        public Vector2 BlendWeights;
        /// <summary>
        /// 影響ボーン1
        /// </summary>
        public int BlendIndexX;
        /// <summary>
        /// 影響ボーン2
        /// </summary>
        public int BlendIndexY;
    }

    /// <summary>
    /// MMD頂点データ(法線付き)
    /// </summary>
    [ContentSerializerRuntimeType("MikuMikuDance.XNA.Model.MMDVertexNm, MikuMikuDanceXNA")]
    public class MMDVertexNmContent : MMDVertexContent
    {
        /// <summary>
        /// 法線
        /// </summary>
        public Vector3 Normal;
    }
    /// <summary>
    /// MMD頂点データ(法線、テクスチャ付き)
    /// </summary>
    [ContentSerializerRuntimeType("MikuMikuDance.XNA.Model.MMDVertexNmTx, MikuMikuDanceXNA")]
    public class MMDVertexNmTxContent : MMDVertexNmContent
    {
        /// <summary>
        /// テクスチャ座標
        /// </summary>
        public Vector2 TextureCoordinate;
    }
    /// <summary>
    /// MMD頂点データ(法線、テクスチャ、頂点カラー付き)
    /// </summary>
    [ContentSerializerRuntimeType("MikuMikuDance.XNA.Model.MMDVertexNmTxVc, MikuMikuDanceXNA")]
    public class MMDVertexNmTxVcContent : MMDVertexNmTxContent
    {
        /// <summary>
        /// 頂点カラー
        /// </summary>
        public Vector4 VertexColor;
    }
    /// <summary>
    /// MMD頂点データ(法線、頂点カラー付き)
    /// </summary>
    [ContentSerializerRuntimeType("MikuMikuDance.XNA.Model.MMDVertexNmTxVc, MikuMikuDanceXNA")]
    public class MMDVertexNmVcContent : MMDVertexNmContent
    {
        /// <summary>
        /// 頂点カラー
        /// </summary>
        public Vector4 VertexColor;
    }
    /// <summary>
    /// MMD頂点データ(テクスチャ付き)
    /// </summary>
    [ContentSerializerRuntimeType("MikuMikuDance.XNA.Model.MMDVertexTx, MikuMikuDanceXNA")]
    public class MMDVertexTxContent : MMDVertexContent
    {
        /// <summary>
        /// テクスチャ座標
        /// </summary>
        public Vector2 TextureCoordinate;
    }
    /// <summary>
    /// MMD頂点データ(テクスチャ、頂点カラー付き)
    /// </summary>
    [ContentSerializerRuntimeType("MikuMikuDance.XNA.Model.MMDVertexTxVc, MikuMikuDanceXNA")]
    public class MMDVertexTxVcContent : MMDVertexTxContent
    {
        /// <summary>
        /// 頂点カラー
        /// </summary>
        public Vector4 VertexColor;
    }
    /// <summary>
    /// MMD頂点データ(頂点カラー付き)
    /// </summary>
    [ContentSerializerRuntimeType("MikuMikuDance.XNA.Model.MMDVertexVc, MikuMikuDanceXNA")]
    public class MMDVertexVcContent : MMDVertexContent
    {
        /// <summary>
        /// 頂点カラー
        /// </summary>
        public Vector4 VertexColor;
    }
}
