﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MikuMikuDance.XNA.Model;
using Microsoft.Xna.Framework.Content;
using MikuMikuDance.XNA.Motion;
using MikuMikuDance.XNA.Misc;

namespace MikuMikuDance.XNA
{
    /// <summary>
    /// MikuMikuDanceXNAコアクラス
    /// </summary>
    public class MMDXCore
    {
        static MMDXCore m_inst;
        /// <summary>
        /// Singletonインスタンス
        /// </summary>
        public static MMDXCore Instance
        {
            get
            {
                if (m_inst == null)
                    m_inst = new MMDXCore();
                return m_inst;
            }
        }
        /// <summary>
        /// MMDXで使用するIKソルバー
        /// </summary>
        public IIKSolver IKSolver { get; set; }
        /// <summary>
        /// MMDXで使用するIKリミッター
        /// </summary>
        public IIKLimitter IKLimitter { get; set; }
        /// <summary>
        /// コンストラクタ
        /// </summary>
        protected MMDXCore()
        {
            //外部からつくらせない
            //IKソルバとリミッター
            IKSolver = new CCDSolver();
            IKLimitter = new DefaltIKLimitter();
        }
        
        /// <summary>
        /// モデルをアセットより読み込む
        /// </summary>
        /// <param name="assetName">アセット名</param>
        /// <param name="content">コンテンツマネージャ</param>
        /// <returns>MMDモデル</returns>
        public MMDModel LoadModel(string assetName, ContentManager content)
        {
            return content.Load<MMDModel>(assetName);
        }
        /// <summary>
        /// モーションをアセットより読み込む
        /// </summary>
        /// <param name="assetName">アセット名</param>
        /// <param name="content">コンテンツマネージャ</param>
        /// <returns>MMDモデル</returns>
        public MMDMotionData LoadMotion(string assetName, ContentManager content)
        {
            return content.Load<MMDMotionData>(assetName);
        }
    }
}
