﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MikuMikuDance.XNA.Misc;
using System.Collections.ObjectModel;
using Microsoft.Xna.Framework;

namespace MikuMikuDance.XNA.Model
{
    /// <summary>
    /// ボーンクラス
    /// </summary>
    public class MMDBone
    {
        /// <summary>
        /// ボーンの名前
        /// </summary>
        public readonly string Name;
        
        /// <summary>
        /// ボーンのバインドポーズ行列
        /// </summary>
        public readonly SQTTransform BindPose;
        /// <summary>
        /// ボーンのバインドポーズ逆行列
        /// </summary>
        /// <remarks>変更しないこと……</remarks>
        public Matrix InverseBindPose;
        /// <summary>
        /// ボーンの親のボーンのインデックス情報
        /// </summary>
        /// <remarks>親が無い場合は-1</remarks>
        public readonly int SkeletonHierarchy;
        /// <summary>
        /// このボーンに対応するIKリスト
        /// </summary>
        public readonly ReadOnlyCollection<int> IKs;

        /// <summary>
        /// ボーンのローカル変換行列
        /// </summary>
        /// <remarks>アニメーション再生時にはアニメーションにより上書きされる</remarks>
        public SQTTransform LocalTransform;

        /// <summary>
        /// ボーンのグローバル変換行列
        /// </summary>
        /// <remarks>毎フレームごとにLocalTransformから計算され、上書きされる</remarks>
        public Matrix GlobalTransform;
        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="name">名前</param>
        /// <param name="bindPose">バインドポーズ</param>
        /// <param name="inverseBindPose">逆バインドポーズ</param>
        /// <param name="skeletonHierarchy">親ボーン番号</param>
        /// <param name="iks">このボーンに対応するIK一覧</param>
        public MMDBone(string name, SQTTransform bindPose, Matrix inverseBindPose, int skeletonHierarchy, List<int> iks)
        {
            Name = name;
            BindPose = bindPose;
            InverseBindPose = inverseBindPose;
            SkeletonHierarchy = skeletonHierarchy;
            IKs = new ReadOnlyCollection<int>(iks);
            LocalTransform = bindPose;
            GlobalTransform = Matrix.Identity;
        }
    }
}
