﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using MikuMikuDance.XNA.Misc;
using Microsoft.Xna.Framework;

namespace MikuMikuDance.XNA.Model
{
    /// <summary>
    /// ボーンマネージャクラス
    /// </summary>
    public class MMDBoneManager
    {
        ReadOnlyCollection<MMDBone> bones;
        /// <summary>
        /// IK一覧
        /// </summary>
        public ReadOnlyCollection<MMDIK> IKs { get; private set; }
        Dictionary<string, int> boneDic;
        internal Matrix[] SkinTransforms { get; private set; }
        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="bones">ボーン一覧</param>
        /// <param name="iks">IK一覧</param>
        public MMDBoneManager(List<MMDBone> bones, List<MMDIK> iks)
        {
            this.bones = new ReadOnlyCollection<MMDBone>(bones);
            this.IKs = new ReadOnlyCollection<MMDIK>(iks);
            boneDic = new Dictionary<string, int>();
            for (int i = 0; i < bones.Count; i++)
            {
                boneDic.Add(bones[i].Name, i);
            }
            SkinTransforms = new Matrix[bones.Count];
        }
        /// <summary>
        /// ボーン取得
        /// </summary>
        /// <param name="index">ボーン番号</param>
        /// <returns>ボーンオブジェクト</returns>
        public MMDBone this[int index] { get { return bones[index]; } }
        /// <summary>
        /// ボーン取得
        /// </summary>
        /// <param name="key">ボーン名</param>
        /// <returns>ボーンオブジェクト</returns>
        public MMDBone this[string key] { get { return bones[boneDic[key]]; } }
        /// <summary>
        /// ボーン番号取得
        /// </summary>
        /// <param name="key">ボーン名</param>
        /// <returns>ボーン番号</returns>
        /// <remarks>ボーン名が存在しない場合は-1が返却される</remarks>
        public int IndexOf(string key)
        {
            int result;
            if (!boneDic.TryGetValue(key, out result))
                return -1;
            return result;
        }
        /// <summary>
        /// ボーン数
        /// </summary>
        public int Count { get { return bones.Count; } }



        internal void CalcGlobalTransform()
        {
            MMDXProfiler.BeginMark("BoneManager.CalcGlobalTransform", new Color(40, 255, 0));
            bones[0].LocalTransform.CreateMatrix(out bones[0].GlobalTransform);
            for (int i = 1; i < bones.Count; ++i)
            {
                int parentBone = bones[i].SkeletonHierarchy;
                Matrix local;
                bones[i].LocalTransform.CreateMatrix(out local);
                Matrix.Multiply(ref local, ref bones[parentBone].GlobalTransform, out bones[i].GlobalTransform);
            }
            MMDXProfiler.EndMark("BoneManager.CalcGlobalTransform");
        }
        

        internal void CalcSkinTransform()
        {
            MMDXProfiler.BeginMark("BoneManager.CalcSkinTransform", new Color(40, 255, 0));
            for (int i = 0; i < bones.Count; ++i)
                Matrix.Multiply(ref bones[i].InverseBindPose, ref bones[i].GlobalTransform, out SkinTransforms[i]);
            MMDXProfiler.EndMark("BoneManager.CalcSkinTransform");
        }
        internal void CalcIK()
        {
            bool UpdateFlag = false;
            foreach (var ik in IKs)
            {
                if (MMDXCore.Instance.IKSolver.Solve(ik, this))
                    UpdateFlag = true;
            }
            if (UpdateFlag)
                CalcGlobalTransform();
        }
    }
}
