﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MikuMikuDance.XNA.Motion;
using System.Collections.ObjectModel;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using MikuMikuDance.XNA.Misc;

namespace MikuMikuDance.XNA.Model
{
    /// <summary>
    /// MMDモデルクラス
    /// </summary>
    public class MMDModel
    {
        private readonly List<MMDModelPart> modelParts;
        //private readonly SkinningData skinningData;
        readonly MMDBoneManager boneManager;
        readonly Dictionary<string, MMDMotionData> attachedMotion;
        private AnimationPlayer animationPlayer;
        /// <summary>
        /// ボーンマネージャ
        /// </summary>
        public MMDBoneManager BoneManager { get { return boneManager; } }
        /// <summary>
        /// アニメーションプレイヤー
        /// </summary>
        public AnimationPlayer AnimationPlayer { get { return animationPlayer; } }

        /// <summary>
        /// このモデルのメッシュパーツ
        /// </summary>
        public ReadOnlyCollection<MMDModelPart> Parts { get; internal set; }
        
        /// <summary>
        /// このモデルのワールド座標
        /// </summary>
        public Matrix Transform;
        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="modelParts">モデルパーツ</param>
        /// <param name="boneManager">ボーンマネージャ</param>
        /// <param name="attachedMotion">付随モーション</param>
        public MMDModel(List<MMDModelPart> modelParts, MMDBoneManager boneManager, Dictionary<string, MMDMotionData> attachedMotion)
        {
            this.modelParts = modelParts;
            this.boneManager = boneManager;
            this.attachedMotion = attachedMotion;

            Parts = new ReadOnlyCollection<MMDModelPart>(modelParts);
            animationPlayer = new AnimationPlayer(boneManager);
        }
        /// <summary>
        /// モデルの更新処理
        /// </summary>
        public void Update()
        {
            MMDXProfiler.BeginMark("ModelUpdate", new Color(20, 255, 0));
            AnimationPlayer.Update();
            BoneManager.CalcGlobalTransform();
            BoneManager.CalcIK();
            BoneManager.CalcSkinTransform();
            MMDXProfiler.BeginMark("ModelPart.SetBone", new Color(60, 255, 0));
            System.Threading.Tasks.Parallel.ForEach(Parts, (part) => part.SetBone(BoneManager.SkinTransforms));
            /*foreach (var part in Parts)
                part.SetBone(BoneManager.SkinTransforms);*/
            MMDXProfiler.EndMark("ModelPart.SetBone");
            MMDXProfiler.EndMark("ModelUpdate");
        }

        /// <summary>
        /// モデルの描画
        /// </summary>
        public void Draw(Matrix view, Matrix projection)
        {
            foreach (var part in modelParts)
            {
                part.SetMatrices(ref Transform, ref view, ref projection);
                //ライティング処理
                IEffectLights lights = part.Effect as IEffectLights;
                if (lights != null)
                {
                    lights.EnableDefaultLighting();
                }

                part.Draw();
            }
        }
        
    }
}
