﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MikuMikuDance.XNA.Model;
using MikuMikuDance.XNA.Misc;
using System.Collections;

namespace MikuMikuDance.XNA.Motion
{
    /// <summary>
    /// モーショントラックの管理、ポーズのブレンディング等を行うクラス
    /// </summary>
    public class AnimationPlayer
    {
        Dictionary<string, MMDMotionTrack> motionTracks = new Dictionary<string, MMDMotionTrack>();
        MMDBoneManager boneManager;
        Dictionary<string, SQTTransform> BindPoses;
        Dictionary<string, SQTTransform> Poses;
        internal AnimationPlayer(MMDBoneManager bones)
        {
            boneManager = bones;
            BindPoses = new Dictionary<string, SQTTransform>();
            for (int i = 0; i < boneManager.Count; ++i)
            {
                BindPoses.Add(boneManager[i].Name, boneManager[i].BindPose);
            }
            Poses = new Dictionary<string, SQTTransform>(BindPoses.Count);
        }

        /// <summary>
        /// モーショントラックの取得
        /// </summary>
        /// <param name="motionKey">モーショントラック名</param>
        /// <returns>モーショントラック</returns>
        public MMDMotionTrack this[string motionKey] 
        { 
            get 
            {
                MMDMotionTrack result = null;
                if (!motionTracks.TryGetValue(motionKey, out result))
                    throw new KeyNotFoundException("モーション名 \"" + motionKey.ToString() + "\" は見つかりません");
                return result;
            } 
        }

        /// <summary>
        /// モーションの追加
        /// </summary>
        /// <param name="motionKey">モーション識別用のモーション名</param>
        /// <param name="motionData">MikuMikuDance MotionData</param>
        public void AddMotion(string motionKey, MMDMotionData motionData)
        {
            motionTracks.Add(motionKey, new MMDMotionTrack(motionData));
        }
        /// <summary>
        /// モーションの追加
        /// </summary>
        /// <param name="motionKey">モーション識別用のモーション名</param>
        /// <param name="motionData">MikuMikuDance MotionTrack</param>
        public void AddMotion(string motionKey, MMDMotionTrack motionData)
        {
            motionTracks.Add(motionKey, motionData);
        }

        /// <summary>
        /// モーションの削除
        /// </summary>
        /// <param name="motionKey"></param>
        public void RemoveMotion(string motionKey)
        {
            motionTracks.Remove(motionKey);
        }

        /// <summary>
        /// 全てのモーションの再生停止
        /// </summary>
        public void StopAll()
        {
            foreach (var motiontrack in motionTracks)
            {
                motiontrack.Value.Stop();
            }
        }
        internal void Update()
        {
            MMDXProfiler.BeginMark("AnimationPlayerUpdate", new Microsoft.Xna.Framework.Color(30, 255, 0));
            //差分ポーズを元にした加算ブレンディング
            Poses.Clear();
            foreach (var track in motionTracks)
            {
                track.Value.Update();
                foreach (var subpose in track.Value.SubPoses)
                {
                    SQTTransform pose1, pose2, sub = subpose.Value, result;
                    if (!Poses.TryGetValue(subpose.Key, out pose1))
                    {
                        if (!BindPoses.TryGetValue(subpose.Key, out pose1))
                            continue;//無いモーションは無視
                    }
                    SQTTransform.Multiply(ref sub, ref pose1, out pose2);
                    SQTTransform.Lerp(ref pose1, ref pose2, track.Value.BoneBlendingFactor, out result);
                    Poses[subpose.Key] = result;
                }
            }
            //ボーンマネージャへの書き戻し
            foreach (var pose in Poses)
            {

                boneManager[pose.Key].LocalTransform = pose.Value;
            }
            MMDXProfiler.EndMark("AnimationPlayerUpdate");
        }

        
    }

    
}
